--[[
AdiFocusBar - Smart focus bar for hunters.
Copyright 2010-2013 Adirelle (adirelle@gmail.com)
All rights reserved.
--]]

if select(2, UnitClass("player")) ~= "HUNTER" then return end

local addonName, ns = ...

local addon, Debug = ns.addon, ns.Debug
local LibMovable = LibStub('LibMovable-1.0')

------------------------------------------------------------------------------
-- Pure GUI part
------------------------------------------------------------------------------

local BAR_BACKDROP = {
	bgFile = [[Interface\Addons\AdiFocusBar\media\white16x16]],
	tile = true,
	tileSize = 16,
}

local BORDER_SIZE = 2
local BORDER_BACKDROP = {
	edgeFile = [[Interface\Addons\AdiFocusBar\media\white16x16]],
	edgeSize = BORDER_SIZE,
}

local BAR_TEXTURE = [[Interface\TargetingFrame\UI-StatusBar]]

local LSM_STATUSBAR = "BantoBar"

local WIDTH, HEIGHT = 250, 15

local function Mark_SetPosition(mark, value, text, icon)
	if value then
		local bar = mark:GetParent()
		local min, max = bar:GetMinMaxValues()
		if value > min and value < max then
			local valueToPixel = bar:GetWidth() / (max - min)
			mark:SetPoint("CENTER", bar, "LEFT", valueToPixel * (value - min), 0)
			mark:Show()
			local t = mark.Text
			if t then
				t:SetShown(not not text)
				if text then
					t:SetText(text)
				end
			end
			local i = mark.Icon
			if i then
				i:SetShown(not not icon)
				if icon then
					i:SetTexture(select(3, GetSpellInfo(icon)))
				end
			end
			return
		end
	end
	mark:Hide()
end

local function Bar_SetValues(bar, newValue, newMax, newMin)
	if newValue and newMax then
		local value, min, max = bar:GetValue(), bar:GetMinMaxValues()
		if min ~= (newMin or 0) or max ~= newMax then
			bar:SetMinMaxValues((newMin or 0), newMax)
		end
		if value ~= newValue then
			bar:SetValue(newValue)
		end
		if not bar:IsShown() then
			bar:Show()
		end
	elseif bar:IsShown() then
		bar:Hide()
	end
end

local function SpawnSpark(bar)
	local spark = bar:CreateTexture(nil, "OVERLAY")
	spark:SetTexture([[Interface\CastingBar\UI-CastingBar-Spark]])
	spark:SetBlendMode('ADD')
	spark:SetSize(20, HEIGHT*2.5)
	spark.SetPosition = Mark_SetPosition
	return spark
end

local function SpawnMark(bar, r, g, b)
	local mark = CreateFrame("Frame", nil, bar)
	mark:SetSize(2, HEIGHT)
	mark.SetPosition = Mark_SetPosition

	local line = mark:CreateTexture(nil, "BACKGROUND")
	line:SetPoint("CENTER")
	line:SetSize(2, HEIGHT)
	line:SetTexture([[Interface\Addons\AdiFocusBar\media\white16x16]])
	line:SetVertexColor(r, g, b)

	local text = mark:CreateFontString(nil, "OVERLAY")
	text:SetPoint("CENTER")
	local name, height, flags = GameFontWhiteSmall:GetFont()
	text:SetFont(name, height, "OUTLINE")
	text:Hide()
	mark.Text = text

	local icon = mark:CreateTexture(nil, "ARTWORK")
	icon:SetPoint("CENTER")
	icon:SetSize(HEIGHT * 0.66, HEIGHT * 0.66)
	icon:Hide()
	mark.Icon = icon

	return mark
end

local function Bar_FixTexture(bar)
	local texture = bar:GetStatusBarTexture()
	local value, min, max = bar:GetValue(), bar:GetMinMaxValues()
	if max ~= min then
		texture:SetTexCoord(0, (value-min)/(max-min), 0, 1)
	end
end

function Bar_OnValueChanged(bar)
	local value = bar:GetValue()
	addon.Text:SetFormattedText("%d", value)
	bar.Spark:SetPosition(value)
	return Bar_FixTexture(bar)
end

function addon:SetForecastIcon(icon)
	local icon1, icon2 = self.Icon1, self.Icon2
	if icon ~= icon1.spell and icon ~= icon2.spell then
		local newIcon = icon1:GetAlpha() < icon2:GetAlpha() and icon1 or icon2
		newIcon:SetTexture(icon and select(3, GetSpellInfo(icon)) or "")
		newIcon.spell = icon
	end
	if icon1.spell == icon and icon1:GetAlpha() < 1 then
		self.fadeInIcon, self.fadeOutIcon = icon1, icon2
	elseif icon2.spell == icon and icon2:GetAlpha() < 1 then
		self.fadeInIcon, self.fadeOutIcon = icon2, icon1
	end
end

function addon:SetupGUI()
	self:SetSize(WIDTH, HEIGHT)
	self:SetBackdrop(BAR_BACKDROP)
	self:SetBackdropColor(0, 0, 0, 0.8)
	self:SetBackdropBorderColor(0, 0, 0, 0)

	local border = CreateFrame("Frame", nil, self)
	border:SetSize(WIDTH+BORDER_SIZE*2, HEIGHT+BORDER_SIZE*2)
	border:SetPoint("CENTER")
	border:SetBackdrop(BORDER_BACKDROP)
	border:SetBackdropColor(0, 0, 0, 0)
	border:SetBackdropBorderColor(0, 0, 0, 1)

	local bar = CreateFrame("StatusBar", nil, self)
	bar:SetPoint("RIGHT")
	bar:SetSize(WIDTH - HEIGHT - 2, HEIGHT)
	bar:SetStatusBarTexture(BAR_TEXTURE)
	bar:SetScript('OnMinMaxChanged', Bar_OnValueChanged)
	bar:SetScript('OnValueChanged', Bar_OnValueChanged)
	bar:SetScript('OnShow', Bar_OnValueChanged)
	bar.SetValues = Bar_SetValues
	self.Bar = bar

	local icon1 = self:CreateTexture(nil, "ARTWORK")
	icon1:SetSize(HEIGHT, HEIGHT)
	icon1:SetPoint("LEFT")
	icon1:SetTexCoord(0.05, 0.95, 0.05, 0.95)
	self.Icon1 = icon1

	local icon2 = self:CreateTexture(nil, "ARTWORK")
	icon2:SetSize(HEIGHT, HEIGHT)
	icon2:SetPoint("LEFT")
	icon2:SetTexCoord(0.05, 0.95, 0.05, 0.95)
	icon2:SetAlpha(0)
	self.Icon2 = icon2

	bar.Spark = SpawnSpark(bar)

	local fillerBar = CreateFrame("StatusBar", nil, self)
	fillerBar:SetAllPoints(bar)
	fillerBar:SetStatusBarTexture(BAR_TEXTURE)
	fillerBar:SetScript('OnMinMaxChanged', Bar_FixTexture)
	fillerBar:SetScript('OnValueChanged', Bar_FixTexture)
	fillerBar.SetValues = Bar_SetValues
	self.FillerBar = fillerBar

	bar:SetFrameLevel(fillerBar:GetFrameLevel()+1)

	local lsm = LibStub('LibSharedMedia-3.0', true)
	if lsm then
		local function Update(value)
			local texture = lsm:Fetch("statusbar", value, true)
			bar:SetStatusBarTexture(texture)
			fillerBar:SetStatusBarTexture(texture)
		end
		Update(LSM_STATUSBAR)

		lsm.RegisterCallback(bar, 'LibSharedMedia_SetGlobal', function(_, _, media, value)
			if media == "statusbar" then
				Update(value)
			end
		end)
	end

	bar:SetStatusBarColor(PowerBarColor.FOCUS.r, PowerBarColor.FOCUS.g, PowerBarColor.FOCUS.b)
	fillerBar:SetStatusBarColor(PowerBarColor.FOCUS.b, PowerBarColor.FOCUS.r, PowerBarColor.FOCUS.g)
	fillerBar:Hide()

	local text = bar:CreateFontString(nil, "OVERLAY", "GameFontWhite")
	text:SetAllPoints(bar)
	text:SetJustifyH("RIGHT")
	text:SetJustifyV("MIDDLE")
	text:SetShadowColor(0,0,0,1)
	text:SetShadowOffset(1, -1)
	self.Text = text

	self.Marks = {
		ArcaneShot = SpawnMark(bar, 1, 0.8, 0.8),
		BlackArrow = SpawnMark(bar, 1, 0, 1),
		ExplosiveShot = SpawnMark(bar, 1, 0.7, 0),
		SerpentSting = SpawnMark(bar, 0, 1, 0),
	}
	self.Marks.KillCommand = self.Marks.ExplosiveShot
	self.Marks.ChimeraShot = self.Marks.ExplosiveShot

	self:SetPoint("BOTTOM", UIParent, 0, 208)
	LibMovable.RegisterMovable(addonName, self, AdiFocusBarDB.layout)

	self:SetScript('OnShow', self.OnShow)
	self:SetScript('OnHide', self.OnHide)
	self:SetScript('OnUpdate', self.OnUpdate)
end

-- Macro command to move the thing around
SlashCmdList["ADIFOCUSBAR"] = function()
	if LibMovable.IsLocked(addonName) then
		LibMovable.Unlock(addonName)
	else
		LibMovable.Lock(addonName)
	end
end
SLASH_ADIFOCUSBAR1 = "/adifocusbar"
SLASH_ADIFOCUSBAR2 = "/afb"
